@@ -1,9 +1,14 @@
 #![feature(test)]
+#![allow(
+    clippy::many_single_char_names,
+    clippy::deref_addrof,
+    clippy::unreadable_literal,
+    clippy::many_single_char_names
+)]
 
 extern crate test;
 use test::Bencher;
 
-extern crate ndarray;
 use ndarray::prelude::*;
 
 use ndarray::linalg::general_mat_vec_mul;
