@@ -20,7 +20,7 @@ use alloc::vec;
 use alloc::vec::Vec;
 
 use crate::dimension;
-use crate::dimension::offset_from_ptr_to_memory;
+use crate::dimension::offset_from_low_addr_ptr_to_logical_ptr;
 use crate::error::{self, ShapeError};
 use crate::extension::nonnull::nonnull_from_vec_data;
 use crate::imp_prelude::*;
@@ -29,6 +29,7 @@ use crate::indices;
 #[cfg(feature = "std")]
 use crate::iterators::to_vec;
 use crate::iterators::to_vec_mapped;
+use crate::iterators::TrustedIterator;
 use crate::StrideShape;
 #[cfg(feature = "std")]
 use crate::{geomspace, linspace, logspace};
@@ -491,10 +492,31 @@ where
         // debug check for issues that indicates wrong use of this constructor
         debug_assert!(dimension::can_index_slice(&v, &dim, &strides).is_ok());
 
-        let ptr = nonnull_from_vec_data(&mut v).offset(-offset_from_ptr_to_memory(&dim, &strides));
+        let ptr = nonnull_from_vec_data(&mut v).add(offset_from_low_addr_ptr_to_logical_ptr(&dim, &strides));
         ArrayBase::from_data_ptr(DataOwned::new(v), ptr).with_strides_dim(strides, dim)
     }
 
+    /// Creates an array from an iterator, mapped by `map` and interpret it according to the
+    /// provided shape and strides.
+    ///
+    /// # Safety
+    ///
+    /// See from_shape_vec_unchecked
+    pub(crate) unsafe fn from_shape_trusted_iter_unchecked<Sh, I, F>(shape: Sh, iter: I, map: F)
+        -> Self
+    where
+        Sh: Into<StrideShape<D>>,
+        I: TrustedIterator + ExactSizeIterator,
+        F: FnMut(I::Item) -> A,
+    {
+        let shape = shape.into();
+        let dim = shape.dim;
+        let strides = shape.strides.strides_for_dim(&dim);
+        let v = to_vec_mapped(iter, map);
+        Self::from_vec_dim_stride_unchecked(dim, strides, v)
+    }
+
+
     /// Create an array with uninitalized elements, shape `shape`.
     ///
     /// The uninitialized elements of type `A` are represented by the type `MaybeUninit<A>`,
@@ -508,7 +530,7 @@ where
     /// ### Safety
     ///
     /// The whole of the array must be initialized before it is converted
-    /// using [`.assume_init()`] or otherwise traversed.
+    /// using [`.assume_init()`] or otherwise traversed/read with the element type `A`.
     ///
     /// ### Examples
     ///
@@ -558,10 +580,10 @@ where
     /// The uninitialized elements of type `A` are represented by the type `MaybeUninit<A>`,
     /// an easier way to handle uninit values correctly.
     ///
-    /// The `builder` closure gets unshared access to the array through a raw view
-    /// and can use it to modify the array before it is returned. This allows initializing
-    /// the array for any owned array type (avoiding clone requirements for copy-on-write,
-    /// because the array is unshared when initially created).
+    /// The `builder` closure gets unshared access to the array through a view and can use it to
+    /// modify the array before it is returned. This allows initializing the array for any owned
+    /// array type (avoiding clone requirements for copy-on-write, because the array is unshared
+    /// when initially created).
     ///
     /// Only *when* the array is completely initialized with valid elements, can it be
     /// converted to an array of `A` elements using [`.assume_init()`].
@@ -571,17 +593,18 @@ where
     /// ### Safety
     ///
     /// The whole of the array must be initialized before it is converted
-    /// using [`.assume_init()`] or otherwise traversed.
+    /// using [`.assume_init()`] or otherwise traversed/read with the element type `A`.
     ///
-    pub(crate) fn build_uninit<Sh, F>(shape: Sh, builder: F) -> ArrayBase<S::MaybeUninit, D>
+    /// [`.assume_init()`]: ArrayBase::assume_init
+    pub fn build_uninit<Sh, F>(shape: Sh, builder: F) -> ArrayBase<S::MaybeUninit, D>
     where
         Sh: ShapeBuilder<Dim = D>,
-        F: FnOnce(RawArrayViewMut<MaybeUninit<A>, D>),
+        F: FnOnce(ArrayViewMut<MaybeUninit<A>, D>),
     {
         let mut array = Self::uninit(shape);
         // Safe because: the array is unshared here
         unsafe {
-            builder(array.raw_view_mut_unchecked());
+            builder(array.raw_view_mut_unchecked().deref_into_view_mut());
         }
         array
     }
