@@ -7,11 +7,14 @@
 // except according to those terms.
 #![crate_name = "ndarray"]
 #![doc(html_root_url = "https://docs.rs/ndarray/0.15/")]
+#![doc(html_logo_url = "https://rust-ndarray.github.io/images/rust-ndarray_logo.svg")]
 #![allow(
     clippy::many_single_char_names,
     clippy::deref_addrof,
     clippy::unreadable_literal,
     clippy::manual_map, // is not an error
+    clippy::while_let_on_iterator, // is not an error
+    clippy::from_iter_instead_of_collect, // using from_iter is good style
 )]
 #![cfg_attr(not(feature = "std"), no_std)]
 
@@ -71,16 +74,16 @@
 //! `Cargo.toml`.
 //!
 //! - `std`
-//!  - Rust standard library (enabled by default)
-//!  - This crate can be used without the standard library by disabling the
-//!    default `std` feature. To do so, use `default-features = false` in
-//!    your `Cargo.toml`.
-//!  - The `geomspace` `linspace` `logspace` `range` `std` `var` `var_axis`
-//!    and `std_axis` methods are only available when `std` is enabled.
+//!   - Rust standard library (enabled by default)
+//!   - This crate can be used without the standard library by disabling the
+//!     default `std` feature. To do so, use `default-features = false` in
+//!     your `Cargo.toml`.
+//!   - The `geomspace` `linspace` `logspace` `range` `std` `var` `var_axis`
+//!     and `std_axis` methods are only available when `std` is enabled.
 //! - `serde`
 //!   - Enables serialization support for serde 1.x
 //! - `rayon`
-//!   - Enables parallel iterators, parallelized methods and [`par_azip!`].
+//!   - Enables parallel iterators, parallelized methods, the [`parallel`] module and [`par_azip!`].
 //!   - Implies std
 //! - `approx`
 //!   - Enables implementations of traits from the [`approx`] crate.
@@ -142,12 +145,13 @@ pub use crate::dimension::IxDynImpl;
 pub use crate::dimension::NdIndex;
 pub use crate::error::{ErrorKind, ShapeError};
 pub use crate::indexes::{indices, indices_of};
+pub use crate::order::Order;
 pub use crate::slice::{
     MultiSliceArg, NewAxis, Slice, SliceArg, SliceInfo, SliceInfoElem, SliceNextDim,
 };
 
 use crate::iterators::Baseiter;
-use crate::iterators::{ElementsBase, ElementsBaseMut, Iter, IterMut, Lanes};
+use crate::iterators::{ElementsBase, ElementsBaseMut, Iter, IterMut};
 
 pub use crate::arraytraits::AsArray;
 #[cfg(feature = "std")]
@@ -159,7 +163,7 @@ pub use crate::stacking::{concatenate, stack, stack_new_axis};
 
 pub use crate::math_cell::MathCell;
 pub use crate::impl_views::IndexLonger;
-pub use crate::shape_builder::{Shape, ShapeBuilder, StrideShape};
+pub use crate::shape_builder::{Shape, ShapeBuilder, ShapeArg, StrideShape};
 
 #[macro_use]
 mod macro_utils;
@@ -199,12 +203,14 @@ mod linspace;
 mod logspace;
 mod math_cell;
 mod numeric_util;
+mod order;
 mod partial;
 mod shape_builder;
 #[macro_use]
 mod slice;
 mod split_at;
 mod stacking;
+mod low_level_util;
 #[macro_use]
 mod zip;
 
@@ -234,8 +240,8 @@ pub type Ixs = isize;
 
 /// An *n*-dimensional array.
 ///
-/// The array is a general container of elements. It cannot grow or shrink, but
-/// can be sliced into subsets of its data.
+/// The array is a general container of elements. It cannot grow or shrink (with some exceptions),
+/// but can be sliced into subsets of its data.
 /// The array supports arithmetic operations by applying them elementwise.
 ///
 /// In *n*-dimensional we include for example 1-dimensional rows or columns,
@@ -1588,12 +1594,6 @@ where
             self.with_strides_dim(s, d)
         }
     }
-
-    /// n-d generalization of rows, just like inner iter
-    fn inner_rows(&self) -> iterators::Lanes<'_, A, D::Smaller> {
-        let n = self.ndim();
-        Lanes::new(self.view(), Axis(n.saturating_sub(1)))
-    }
 }
 
 // parallel methods
