@@ -17,7 +17,6 @@
 //! ```
 
 #[doc(no_inline)]
-#[allow(deprecated)]
 pub use crate::{
     ArcArray, Array, ArrayBase, ArrayView, ArrayViewMut, CowArray, RawArrayView, RawArrayViewMut,
 };
@@ -51,4 +50,11 @@ pub use crate::{array, azip, s};
 pub use crate::ShapeBuilder;
 
 #[doc(no_inline)]
-pub use crate::{AsArray, NdFloat};
+pub use crate::NewAxis;
+
+#[doc(no_inline)]
+pub use crate::AsArray;
+
+#[doc(no_inline)]
+#[cfg(feature = "std")]
+pub use crate::NdFloat;
