@@ -20,8 +20,8 @@ use crate::Layout;
 use crate::partial::Partial;
 
 use crate::indexes::{indices, Indices};
-use crate::layout::{CORDER, FORDER};
 use crate::split_at::{SplitPreference, SplitAt};
+use crate::dimension;
 
 pub use self::ndproducer::{NdProducer, IntoNdProducer, Offset};
 
@@ -52,33 +52,38 @@ where
     private_decl! {}
 }
 
+/// Compute `Layout` hints for array shape dim, strides
+fn array_layout<D: Dimension>(dim: &D, strides: &D) -> Layout {
+    let n = dim.ndim();
+    if dimension::is_layout_c(dim, strides) {
+        // effectively one-dimensional => C and F layout compatible
+        if n <= 1 || dim.slice().iter().filter(|&&len| len > 1).count() <= 1 {
+            Layout::one_dimensional()
+        } else {
+            Layout::c()
+        }
+    } else if n > 1 && dimension::is_layout_f(dim, strides) {
+        Layout::f()
+    } else if n > 1 {
+        if dim[0] > 1 && strides[0] == 1 {
+            Layout::fpref()
+        } else if dim[n - 1] > 1 && strides[n - 1] == 1 {
+            Layout::cpref()
+        } else {
+            Layout::none()
+        }
+    } else {
+        Layout::none()
+    }
+}
+
 impl<S, D> ArrayBase<S, D>
 where
     S: RawData,
     D: Dimension,
 {
     pub(crate) fn layout_impl(&self) -> Layout {
-        let n = self.ndim();
-        if self.is_standard_layout() {
-            // effectively one-dimensional => C and F layout compatible
-            if n <= 1 || self.shape().iter().filter(|&&len| len > 1).count() <= 1 {
-                Layout::one_dimensional()
-            } else {
-                Layout::c()
-            }
-        } else if n > 1 && self.raw_view().reversed_axes().is_standard_layout() {
-            Layout::f()
-        } else if n > 1 {
-            if self.stride_of(Axis(0)) == 1 {
-                Layout::fpref()
-            } else if self.stride_of(Axis(n - 1)) == 1 {
-                Layout::cpref()
-            } else {
-                Layout::none()
-            }
-        } else {
-            Layout::none()
-        }
+        array_layout(&self.dim, &self.strides)
     }
 }
 
@@ -240,22 +245,25 @@ where
     }
 }
 
+#[inline]
+fn zip_dimension_check<D, P>(dimension: &D, part: &P)
+where
+    D: Dimension,
+    P: NdProducer<Dim = D>,
+{
+    ndassert!(
+        part.equal_dim(&dimension),
+        "Zip: Producer dimension mismatch, expected: {:?}, got: {:?}",
+        dimension,
+        part.raw_dim()
+    );
+}
+
+
 impl<Parts, D> Zip<Parts, D>
 where
     D: Dimension,
 {
-    fn check<P>(&self, part: &P)
-    where
-        P: NdProducer<Dim = D>,
-    {
-        ndassert!(
-            part.equal_dim(&self.dimension),
-            "Zip: Producer dimension mismatch, expected: {:?}, got: {:?}",
-            self.dimension,
-            part.raw_dim()
-        );
-    }
-
     /// Return a the number of element tuples in the Zip
     pub fn size(&self) -> usize {
         self.dimension.size()
@@ -269,7 +277,8 @@ where
     }
 
     fn prefer_f(&self) -> bool {
-        !self.layout.is(CORDER) && (self.layout.is(FORDER) || self.layout_tendency < 0)
+        !self.layout.is(Layout::CORDER) &&
+            (self.layout.is(Layout::FORDER) || self.layout_tendency < 0)
     }
 
     /// Return an *approximation* to the max stride axis; if
@@ -307,7 +316,7 @@ where
     {
         if self.dimension.ndim() == 0 {
             function(acc, unsafe { self.parts.as_ref(self.parts.as_ptr()) })
-        } else if self.layout.is(CORDER | FORDER) {
+        } else if self.layout.is(Layout::CORDER | Layout::FORDER) {
             self.for_each_core_contiguous(acc, function)
         } else {
             self.for_each_core_strided(acc, function)
@@ -319,7 +328,7 @@ where
         F: FnMut(Acc, P::Item) -> FoldWhile<Acc>,
         P: ZippableTuple<Dim = D>,
     {
-        debug_assert!(self.layout.is(CORDER | FORDER));
+        debug_assert!(self.layout.is(Layout::CORDER | Layout::FORDER));
         let size = self.dimension.size();
         let ptrs = self.parts.as_ptr();
         let inner_strides = self.parts.contiguous_stride();
@@ -427,6 +436,26 @@ where
     }
 }
 
+impl<D, P1, P2> Zip<(P1, P2), D>
+where
+    D: Dimension,
+    P1: NdProducer<Dim=D>,
+    P1: NdProducer<Dim=D>,
+{
+    /// Debug assert traversal order is like c (including 1D case)
+    // Method placement: only used for binary Zip at the moment.
+    #[inline]
+    pub(crate) fn debug_assert_c_order(self) -> Self {
+        debug_assert!(self.layout.is(Layout::CORDER) || self.layout_tendency >= 0 ||
+                      self.dimension.slice().iter().filter(|&&d| d > 1).count() <= 1,
+                      "Assertion failed: traversal is not c-order or 1D for \
+                      layout {:?}, tendency {}, dimension {:?}",
+                      self.layout, self.layout_tendency, self.dimension);
+        self
+    }
+}
+
+
 /*
 trait Offset : Copy {
     unsafe fn offset(self, off: isize) -> Self;
@@ -652,10 +681,30 @@ macro_rules! map_impl {
                 where P: IntoNdProducer<Dim=D>,
             {
                 let part = p.into_producer();
-                self.check(&part);
+                zip_dimension_check(&self.dimension, &part);
                 self.build_and(part)
             }
 
+            /// Include the producer `p` in the Zip.
+            ///
+            /// ## Safety
+            ///
+            /// The caller must ensure that the producer's shape is equal to the Zip's shape.
+            /// Uses assertions when debug assertions are enabled.
+            #[allow(unused)]
+            pub(crate) unsafe fn and_unchecked<P>(self, p: P) -> Zip<($($p,)* P::Output, ), D>
+                where P: IntoNdProducer<Dim=D>,
+            {
+                #[cfg(debug_assertions)]
+                {
+                    self.and(p)
+                }
+                #[cfg(not(debug_assertions))]
+                {
+                    self.build_and(p.into_producer())
+                }
+            }
+
             /// Include the producer `p` in the Zip, broadcasting if needed.
             ///
             /// If their shapes disagree, `rhs` is broadcast to the shape of `self`.
@@ -702,7 +751,7 @@ macro_rules! map_impl {
                     // Use partial to count the number of filled elements, and can drop the right
                     // number of elements on unwinding (if it happens during apply/collect).
                     unsafe {
-                        let output_view = output.cast::<R>();
+                        let output_view = output.into_raw_view_mut().cast::<R>();
                         self.and(output_view)
                             .collect_with_partial(f)
                             .release_ownership();
@@ -796,7 +845,7 @@ macro_rules! map_impl {
                     // debug assert that the output is contiguous in the memory layout we need
                     if cfg!(debug_assertions) {
                         let out_layout = output.layout();
-                        assert!(out_layout.is(CORDER | FORDER));
+                        assert!(out_layout.is(Layout::CORDER | Layout::FORDER));
                         assert!(
                             (self.layout_tendency <= 0 && out_layout.tendency() <= 0) ||
                             (self.layout_tendency >= 0 && out_layout.tendency() >= 0),
