@@ -4,17 +4,16 @@
     clippy::unreadable_literal,
     clippy::many_single_char_names
 )]
-
+#![cfg(feature = "std")]
 use ndarray::linalg::general_mat_mul;
 use ndarray::prelude::*;
 use ndarray::{rcarr1, rcarr2};
 use ndarray::{Data, LinalgScalar};
 use ndarray::{Ix, Ixs};
-use std::iter::FromIterator;
+use num_traits::Zero;
 
 use approx::assert_abs_diff_eq;
 use defmac::defmac;
-use std::ops::Neg;
 
 fn test_oper(op: &str, a: &[f32], b: &[f32], c: &[f32]) {
     let aa = rcarr1(a);
@@ -33,10 +32,9 @@ fn test_oper(op: &str, a: &[f32], b: &[f32], c: &[f32]) {
     test_oper_arr::<f32, _>(op, aa.clone(), bb.clone(), cc.clone());
 }
 
-fn test_oper_arr<A, D>(op: &str, mut aa: ArcArray<A, D>, bb: ArcArray<A, D>, cc: ArcArray<A, D>)
+
+fn test_oper_arr<A, D>(op: &str, mut aa: ArcArray<f32, D>, bb: ArcArray<f32, D>, cc: ArcArray<f32, D>)
 where
-    A: NdFloat,
-    for<'a> &'a A: Neg<Output = A>,
     D: Dimension,
 {
     match op {
@@ -147,17 +145,16 @@ fn scalar_operations() {
     }
 }
 
-fn reference_dot<'a, A, V1, V2>(a: V1, b: V2) -> A
+fn reference_dot<'a, V1, V2>(a: V1, b: V2) -> f32
 where
-    A: NdFloat,
-    V1: AsArray<'a, A>,
-    V2: AsArray<'a, A>,
+    V1: AsArray<'a, f32>,
+    V2: AsArray<'a, f32>,
 {
     let a = a.into();
     let b = b.into();
     a.iter()
         .zip(b.iter())
-        .fold(A::zero(), |acc, (&x, &y)| acc + x * y)
+        .fold(f32::zero(), |acc, (&x, &y)| acc + x * y)
 }
 
 #[test]
@@ -564,7 +561,8 @@ fn scaled_add_2() {
 #[test]
 fn scaled_add_3() {
     use approx::assert_relative_eq;
-    use ndarray::{SliceInfo, SliceOrIndex};
+    use ndarray::{Slice, SliceInfo, SliceInfoElem};
+    use std::convert::TryFrom;
 
     let beta = -2.3;
     let sizes = vec![
@@ -585,12 +583,12 @@ fn scaled_add_3() {
                 let mut a = range_mat64(m, k);
                 let mut answer = a.clone();
                 let cdim = if n == 1 { vec![q] } else { vec![n, q] };
-                let cslice = if n == 1 {
-                    vec![SliceOrIndex::from(..).step_by(s2)]
+                let cslice: Vec<SliceInfoElem> = if n == 1 {
+                    vec![Slice::from(..).step_by(s2).into()]
                 } else {
                     vec![
-                        SliceOrIndex::from(..).step_by(s1),
-                        SliceOrIndex::from(..).step_by(s2),
+                        Slice::from(..).step_by(s1).into(),
+                        Slice::from(..).step_by(s2).into(),
                     ]
                 };
 
@@ -598,7 +596,7 @@ fn scaled_add_3() {
 
                 {
                     let mut av = a.slice_mut(s![..;s1, ..;s2]);
-                    let c = c.slice(SliceInfo::<_, IxDyn>::new(cslice).unwrap().as_ref());
+                    let c = c.slice(SliceInfo::<_, IxDyn, IxDyn>::try_from(cslice).unwrap());
 
                     let mut answerv = answer.slice_mut(s![..;s1, ..;s2]);
                     answerv += &(beta * &c);
@@ -669,17 +667,21 @@ fn gemm_64_1_f() {
 fn gen_mat_mul_i32() {
     let alpha = -1;
     let beta = 2;
-    let sizes = vec![
-        (4, 4, 4),
-        (8, 8, 8),
-        (17, 15, 16),
-        (4, 17, 3),
-        (17, 3, 22),
-        (19, 18, 2),
-        (16, 17, 15),
-        (15, 16, 17),
-        (67, 63, 62),
-    ];
+    let sizes = if cfg!(miri) {
+        vec![(4, 4, 4), (4, 7, 3)]
+    } else {
+        vec![
+            (4, 4, 4),
+            (8, 8, 8),
+            (17, 15, 16),
+            (4, 17, 3),
+            (17, 3, 22),
+            (19, 18, 2),
+            (16, 17, 15),
+            (15, 16, 17),
+            (67, 63, 62),
+        ]
+    };
     for &(m, k, n) in &sizes {
         let a = range_i32(m, k);
         let b = range_i32(k, n);
@@ -708,7 +710,7 @@ fn gen_mat_vec_mul() {
         S2: Data<Elem = A>,
     {
         let ((m, _), k) = (lhs.dim(), rhs.dim());
-        reference_mat_mul(lhs, &rhs.to_owned().into_shape((k, 1)).unwrap())
+        reference_mat_mul(lhs, &rhs.as_standard_layout().into_shape((k, 1)).unwrap())
             .into_shape(m)
             .unwrap()
     }
@@ -773,7 +775,7 @@ fn vec_mat_mul() {
         S2: Data<Elem = A>,
     {
         let (m, (_, n)) = (lhs.dim(), rhs.dim());
-        reference_mat_mul(&lhs.to_owned().into_shape((1, m)).unwrap(), rhs)
+        reference_mat_mul(&lhs.as_standard_layout().into_shape((1, m)).unwrap(), rhs)
             .into_shape(n)
             .unwrap()
     }
